"use strict";

var _interopRequireWildcard = require("babel-runtime/helpers/interop-require-wildcard")["default"];

exports.__esModule = true;
exports._printClass = _printClass;
exports.ClassDeclaration = ClassDeclaration;
exports.ClassExpression = ClassExpression;
exports.ClassBody = ClassBody;
exports.ClassProperty = ClassProperty;
exports.MethodDefinition = MethodDefinition;
exports.ClassMethod = ClassMethod;

var _babelTypes = require("babel-types");

var t = _interopRequireWildcard(_babelTypes);

function _printClass(node) {
  this.printMultiple(node, "decorators", { separator: null });
  if (node.decorators && node.decorators.length) this.newline();
  this.push("class");

  if (node.id) {
    this.print(node, "id");
  }

  if (node.typeParameters) this.print(node, "typeParameters");

  if (node.superClass) {
    this.push("extends");
    this.print(node, "superClass");
    if (node.superTypeParameters) this.print(node, "superTypeParameters");
  }

  if (node["implements"]) {
    this.push("implements");
    this.printMultiple(node, "implements", { separator: "," });
  }

  if (node.body.leadingSeparators) {
    if (!node.body.body.length) this.push("then");
    for (var i = node.body.leadingSeparators; i > 0; --i) {
      this.push(";;");
    }
  }

  // NOTE: this is a classBody, not a block
  this.print(node, "body");
}

function ClassDeclaration(node, parent) {
  this._printClass(node);
  this.newline();
}

function ClassExpression(node, parent) {
  // TODO: check this.format.parenthesizeClassExpressions
  var needsParens = t.isClass(parent) && parent.superClass === node;
  if (needsParens) this.push("(");
  this._printClass(node);
  if (needsParens) this.push(")");
}

function ClassBody(node) {
  this.indent();
  // TODO: this.printInnerComments(node);
  if (node.body.length === 0) {
    // this.push("pass");
  } else {
      this.newline();
      this.printStatements(node, "body");
    }
  this.dedent();
}

function ClassProperty(node) {
  this.printMultiple(node, "decorators", { separator: null });

  if (node["static"]) this.push("static");
  this.print(node, "key");
  if (node.typeAnnotation) this.print(node, "typeAnnotation");
  if (node.value) {
    this.push("=");
    this.print(node, "value");
  }
  this.newline();
}

function MethodDefinition(node) {
  this.printMultiple(node, "decorators", { separator: null });

  if (node["static"]) {
    this.push("static");
  }

  this._method(node);
  this.newline();
}

function ClassMethod(node) {
  this.printMultiple(node, "decorators", { separator: null });

  if (node["static"]) {
    this.push("static");
  }

  if (node.kind === "constructorCall") {
    this.push({ type: "name", value: { value: "call", raw: "call" } });
  }

  this._method(node, true);
  this.newline();
}
//# sourceMappingURL=classes.js.map
